@@ -32,7 +32,7 @@ from typing import Tuple  # noqa: F401
 
 import launch.logging
 
-import osrf_pycommon.process_utils
+import osrf_pycommon
 
 from .event import Event
 from .event_handlers import OnIncludeLaunchDescription
@@ -42,7 +42,7 @@ from .events import Shutdown
 from .launch_context import LaunchContext
 from .launch_description import LaunchDescription
 from .launch_description_entity import LaunchDescriptionEntity
-from .some_actions_type import SomeActionsType
+from .some_entities_type import SomeEntitiesType
 from .utilities import AsyncSafeSignalManager
 from .utilities import visit_all_entities_and_collect_futures
 
@@ -159,7 +159,7 @@ class LaunchService:
                     raise RuntimeError(
                         'LaunchService cannot be run multiple times concurrently.'
                     )
-                this_loop = asyncio.get_event_loop()
+                this_loop = osrf_pycommon.process_utils.get_loop()
 
                 if self.__debug:
                     this_loop.set_debug(True)
@@ -264,6 +264,7 @@ class LaunchService:
         asynchronous runs.
 
         :param: shutdown_when_idle if True (default), the service will shutdown when idle.
+        :return: the return code (non-zero if there are any errors)
         """
         # Make sure this has not been called from any thread but the main thread.
         if threading.current_thread() is not threading.main_thread():
@@ -290,7 +291,9 @@ class LaunchService:
                     # the queue
                     is_idle = self._is_idle()  # self._entity_future_pairs is pruned here
                     if not self.__shutting_down and shutdown_when_idle and is_idle:
-                        ret = await self._shutdown(reason='idle', due_to_sigint=False)
+                        ret = self._shutdown(reason='idle', due_to_sigint=False)
+                        if ret is not None:
+                            ret = await ret
                         assert ret is None, ret
                         continue
 
@@ -343,7 +346,9 @@ class LaunchService:
                     msg = 'Caught exception in launch (see debug for traceback): {}'.format(exc)
                     self.__logger.debug(traceback.format_exc())
                     self.__logger.error(msg)
-                    ret = await self._shutdown(reason=msg, due_to_sigint=False)
+                    ret = self._shutdown(reason=msg, due_to_sigint=False)
+                    if ret is not None:
+                        ret = await ret
                     assert ret is None, ret
                     return_code = 1
                     # keep running to let things shutdown properly
@@ -361,6 +366,7 @@ class LaunchService:
         After the run ends, this behavior is undone.
 
         :param: shutdown_when_idle if True (default), the service will shutdown when idle
+        :return: the return code (non-zero if there are any errors)
         """
         loop = osrf_pycommon.process_utils.get_loop()
         run_async_task = loop.create_task(self.run_async(
@@ -372,7 +378,7 @@ class LaunchService:
             except KeyboardInterrupt:
                 continue
 
-    def __on_shutdown(self, event: Event, context: LaunchContext) -> Optional[SomeActionsType]:
+    def __on_shutdown(self, event: Event, context: LaunchContext) -> Optional[SomeEntitiesType]:
         self.__shutting_down = True
         self.__context._set_is_shutdown(True)
         return None
@@ -384,7 +390,7 @@ class LaunchService:
             shutdown_event = Shutdown(reason=reason, due_to_sigint=due_to_sigint)
             asyncio_event_loop = None
             try:
-                asyncio_event_loop = asyncio.get_event_loop()
+                asyncio_event_loop = osrf_pycommon.process_utils.get_loop()
             except (RuntimeError, AssertionError):
                 # If no event loop is set for this thread, asyncio will raise an exception.
                 # The exception type depends on the version of Python, so just catch both.
@@ -419,6 +425,7 @@ class LaunchService:
                     reason='LaunchService.shutdown() called',
                     due_to_sigint=False, force_sync=force_sync
                 )
+        return None
 
     @property
     def context(self):
